/*
*******************************************************************************
**
**  This device driver was created by Applilet2 for 78K0/Kx2
**  8-Bit Single-Chip Microcontrollers
**
**  Copyright(C) NEC Electronics Corporation 2002 - 2012
**  All rights reserved by NEC Electronics Corporation.
**
**  This program should be used on your own responsibility.
**  NEC Electronics Corporation assumes no responsibility for any losses
**  incurred by customers or third parties arising from the use of this file.
**
**  Filename :	Serial.h
**  Abstract :	This file implements device driver for Serial module.
**  APIlib :	Applilet2 for 78K0/Kx2 V2.41 [29 Apr. 2008]
**
**  Device :	uPD78F0515_48
**
**  Compiler :	CC78K0
**
**  Creation date:	2012/07/11
**  
*******************************************************************************
*/
#ifndef _MDSERIAL_
#define _MDSERIAL_
/*
*******************************************************************************
**	Register bit define
*******************************************************************************
*/
/*
*******************************************************************************
**	UART0
*******************************************************************************
*/
/* Asynchronous serial interface operation mode register (ASIM0) */
#define UART0_ASIM0_INITIALVALUE	0x01

/* Enable/disable UART0 operation of internal operation clock(POWER0) */
#define	UART0_POWER_ON		0x80	/* enable operation of the internal operation clock */
#define	UART0_POWER_OFF		0x00	/* disable operation of the internal operation clock */

/* UART0 transfer parity(PS01,PS00) */
#define	UART0_PARITY_NONE	0x00		/* transfer without parity */
#define	UART0_PARITY_ZERO	0x08		/* judges as 0 parity */
#define	UART0_PARITY_ODD	0x10		/* judges as odd parity */
#define	UART0_PARITY_EVEN	0x18		/* judges as even parity */
										
/* UART0 character length of transfer data(CL0) */
#define	UART0_DATA_LENGTH_8	0x04		/* character length of data = 8 bits */
#define	UART0_DATA_LENGTH_7	0x00		/* character length of data = 7 bits */

/* UART0 stop bits of transmit data(SL0) */
#define	UART0_STOP_BIT_2	0x02		/* stop bits = 2 */
#define	UART0_STOP_BIT_1	0x00		/* stop bits = 1 */

/* Asynchronous	serial interface Receive error status register (ASIS0) */
/* Status flag indicating parity error(PE0) */
#define	UART0_PARITY_ERROR	0x04
/* Status flag indicating framing error(FE0) */
#define	UART0_FRAMING_ERROR	0x02
/* Status flag indicating overrun error(OVE0) */
#define	UART0_OVERRUN_ERROR	0x01

/* Format of Baud Rate Generator Control Register 0 (BRGC0) */
/* Base clock selection (TPS01,TPS00) */
#define	UART0_BASECLK_1		0x00		/* fxclk0 = TM50 output */
#define	UART0_BASECLK_2		0x40		/* fxclk0 = fprs/2 */
#define	UART0_BASECLK_3		0x80		/* fxclk0 = fprs/8 */
#define	UART0_BASECLK_4		0xc0		/* fxclk0 = fprs/32 */

/*
 Asynchronous serial interface operation mode register (ASIM6n)
*/
/* Enable/disable UART6 operation of internal operation clock(POWER6n) */
#define UART6_POWER_ON		0x80	/* enable operation of the internal operation clock */
#define UART6_POWER_OFF		0x00	/* disable operation of the internal operation clock */
/*uart6 transfer parity(PS61,PS60)*/
#define UART6_PARITY_NONE	0x00	/* transfer without parity */
#define UART6_PARITY_ZERO	0x08	/* judges as 0 parity */
#define UART6_PARITY_ODD	0x10	/* judges as odd parity */
#define UART6_PARITY_EVEN	0x18	/* judges as even parity */
/* uart6 character length of transfer data(CL6) */
#define UART6_DATA_LENGTH_8	0x04	/* character length of data = 8 bits */
#define UART6_DATA_LENGTH_7	0x00	/* character length of data = 7 bits */
/* uart6 stop bits of transmit data(SL6) */
#define UART6_STOP_BIT_2	0x02	/* stop bits = 2 */
#define UART6_STOP_BIT_1	0x00	/* stop bits = 1 */
/* uart6 occurrence of reception completion interrupt in case of error(ISRM6) */
#define UART6_ERR_INTSRE6	0x00	/* INTSRE6 occurs in case of error */
#define UART6_ERR_INTSR6	0x01	/* INTSR6 occurs in case of error */
/* Asynchronous serial interface reception error status register (ASIS6) */
/* Status flag indicating parity error(PE6) */
#define UART6_PARITY_ERROR	0x04
/* Status flag indicating framing error(FE6) */
#define UART6_FRAMING_ERROR	0x02
/* Status flag indicating overrun error(OVE6) */
#define UART6_OVERRUN_ERROR	0x01
/* Format of baud rate generator control register 0 (CKSR6) */
#define UART6_BASECLK_1		0x00	/* fxclk0 = fprs */
#define UART6_BASECLK_2		0x01	/* fxclk0 = fprs/2 */
#define UART6_BASECLK_3		0x02	/* fxclk0 = fprs/4 */
#define UART6_BASECLK_4		0x03	/* fxclk0 = fprs/8 */
#define UART6_BASECLK_5		0x04	/* fxclk0 = fprs/16 */
#define UART6_BASECLK_6		0x05	/* fxclk0 = fprs/32 */
#define UART6_BASECLK_7		0x06	/* fxclk0 = fprs/64 */
#define UART6_BASECLK_8		0x07	/* fxclk0 = fprs/128 */
#define UART6_BASECLK_9		0x08	/* fxclk0 = fprs/256 */
#define UART6_BASECLK_10	0x09	/* fxclk0 = fprs/512 */
#define UART6_BASECLK_11	0x0a	/* fxclk0 = fprs/1024 */
#define UART6_BASECLK_12	0x0b	/* fxclk0 = TM50 output */

/*
 Format of asynchronous serial interface control register 6 (ASICL6)
*/
#define UART6_ASICL6_INITIALVALUE	0x14	
/* First-bit direction(DIR6) */
#define UART6_FIRST_BIT_LSB			0x02	/* LSB */
#define UART6_FIRST_BIT_MSB			0x00	/* MSB */
/* Enable/disable inverting TxD6 output(TXDLV6) */
#define UART6_TXD6_OUTPUT_INVERTED	0x01	/* TxD6 output inverted */
#define UART6_TXD6_OUTPUT_NORMAL	0x00	/* TxD6 output normal */
/* Format of asynchronous serial interface transmission status register 6 (ASIF6) */
#define UART6_TRANSMIT_BUFFER_FLAG	0x02	/* if data exists in TXB6,the data is written to transmit buffer register 6 (TXB6) */
#define UART6_TRANSMIT_SHIFT_FLAG	0x01	/* if data transmission is in progress,the data is transferred from transmit buffer register 6 (TXB6) */
/*
	Serial operation mode register (CSIM1n)
*/

/* Receive/transfer mode control (TRMD1n) */
#define	CSI1_MODE_RECEIVE	0x00	/* receive mode */
#define	CSI1_MODE_BOTH		0x40	/* transmit/receive mode */
/* First bit specification (DIR1n) */
#define	CSI1_FIRST_BIT_MSB	0x00	/* MSB */
#define	CSI1_FIRST_BIT_LSB	0x10	/* LSB */
/* SSI11 pin use selection (SSE11) */
#define	CSI1_SSI11_UNUSED	0x00	/* SSI11 pin is not used */
#define	CSI1_SSI11_USED		0x20	/* SSI11 pin is used */

/*
	Serial clock selection register (CSIC1n)
*/
/* Receive/transmit timing (CKP1n,DAP1n) */
#define	CSI1_DATA_TIMING_1	0x00	/* type 1 */
#define	CSI1_DATA_TIMING_2	0x08	/* type 2 */
#define	CSI1_DATA_TIMING_3	0x10	/* type 3 */
#define	CSI1_DATA_TIMING_4	0x18	/* type 4 */
/* Clock selection (CKS1n2,CKS1n1,CKS1n0) */
#define	CSI1_CLOCK_1		0x00	/* fprs/2 */
#define	CSI1_CLOCK_2		0x01	/* fprs/4 */
#define	CSI1_CLOCK_3		0x02	/* fprs/8 */
#define	CSI1_CLOCK_4		0x03	/* fprs/16 */
#define	CSI1_CLOCK_5		0x04	/* fprs/32 */
#define	CSI1_CLOCK_6		0x05	/* fprs/64 */
#define	CSI1_CLOCK_7		0x06	/* fprs/128 */
#define	CSI1_CLOCK_EXT		0x07	/* external clock */
/*
	IIC control register 0 (IICC0)
*/
/* IIC operation enable (IICE0)	*/
#define	IIC0_OPERATION				0x80
#define	IIC0_OPERATION_DISABLE		0x00	/* stop	operation */
#define	IIC0_OPERATION_ENABLE		0x80	/* enable operation	*/

/* Exit from communications (LREL0)	*/
#define	IIC0_COMMUNICATION			0x40
#define	IIC0_COMMUNICATION_NORMAL	0x00	/* normal operation	*/
#define	IIC0_COMMUNICATION_EXIT		0x40	/* exit	from current communication */

/* Wait cancellation (WREL0) */
#define	IIC0_WAITCANCEL				0x20
#define	IIC0_WAIT_NOTCANCEL			0x00	/* do not cancel wait */
#define	IIC0_WAIT_CANCEL			0x20	/* cancel wait */

/* Generation of interrupt when stop condition (SPIE0) */
#define	IIC0_STOPINT				0x10
#define	IIC0_STOPINT_DISABLE		0x00	/* disable */
#define	IIC0_STOPINT_ENABLE			0x10	/* enable */

/* Wait and interrupt generation (WTIM0) */
#define	IIC0_WAITINT				0x08
#define	IIC0_WAITINT_CLK8FALLING	0x00	/* generate	at the eighth clock's falling edge	*/	 
#define	IIC0_WAITINT_CLK9FALLING	0x08	/* generated at	the	ninth clock's falling edge	*/ 

/* Acknowledgement control (ACKE0) */
#define	IIC0_ACK					0x04
#define	IIC0_ACK_DISABLE			0x00	/* enable acknowledgement */
#define	IIC0_ACK_ENABLE				0x04	/* disable acknowledgement */

/* Start condition trigger (STT0) */
#define	IIC0_STARTCONDITION			0x02
#define	IIC0_START_NOTGENERATE		0x00	/* do not generate start condition */
#define	IIC0_START_GENERATE			0x02	/* generate	start condition	*/

/* Stop condition trigger (SPT0) */
#define	IIC0_STOPCONDITION			0x01
#define	IIC0_STOP_NOTGENERATE		0x00	/* do not generate stop	condition */
#define	IIC0_STOP_GENERATE			0x01	/* generate	stop condition */

/*
	IIC Status Register 0 (IICS0)
*/
/* Master device status (MSTS0)	*/
#define	IIC0_MASTERSTATUS			0x80
#define	IIC0_STATUS_NOTMASTER		0x00	/* slave device	status or communication	standby	status */
#define	IIC0_STATUS_MASTER			0x80	/* master device communication status */

/* Detection of arbitration loss (ALD0)	*/
#define	IIC0_ARBITRATION			0x40
#define	IIC0_ARBITRATION_NO			0x00	/* arbitration win or no arbitration */
#define	IIC0_ARBITRATION_LOSS		0x40	/* arbitration loss	*/

/* Detection of extension code reception (EXC0)	*/
#define	IIC0_EXTENSIONCODE			0x20
#define	IIC0_EXTCODE_NOT			0x00	/* extension code not received */
#define	IIC0_EXTCODE_RECEIVED		0x20	/* extension code received */

/* Detection of matching addresses (COI0) */
#define	IIC0_ADDRESSMATCH			0x10
#define	IIC0_ADDRESS_NOTMATCH		0x00	/* addresses do	not	match */
#define	IIC0_ADDRESS_MATCH			0x10	/* addresses match */

/* Detection of transmit/receive status	(TRC0) */
#define	IIC0_STATUS					0x08
#define	IIC0_STATUS_RECEIVE			0x00	/* receive status */ 
#define	IIC0_STATUS_TRANSMIT		0x08	/* transmit	status */

/* Detection of acknowledge signal (ACKD0) */
#define	IIC0_ACKDETECTION			0x04
#define	IIC0_ACK_NOTDETECTED		0x00	/* ACK signal was not detected */
#define	IIC0_ACK_DETECTED			0x04	/* ACK signal was detected */

/* Detection of start condition (STD0) */
#define	IIC0_STARTDETECTION			0x02
#define	IIC0_START_NOTDETECTED		0x00	/* start condition not detected	*/
#define	IIC0_START_DETECTED			0x02	/* start condition detected	*/

/* Detection of stop condition (SPD0) */
#define	IIC0_STOPDETECTION			0x01
#define	IIC0_STOP_NOTDETECTED		0x00	/* stop	condition not detected */
#define	IIC0_STOP_DETECTED			0x01	/* stop	condition detected */

/*
	IIC Flag Register 0 (IICF0)
*/
/* STT0 clear flag (STCF) */
#define	IIC0_STARTFLAG				0x80
#define	IIC0_STARTFLAG_GENERATE		0x00	/* generate	start condition	*/
#define	IIC0_STARTFLAG_UNSUCCESSFUL	0x80	/* start condition generation unsuccessful */

/* IIC bus status flag (IICBSY) */
#define	IIC0_BUSSTATUS				0x40
#define	IIC0_BUS_RELEASE			0x00	/* bus release status */
#define	IIC0_BUS_COMMUNICATION		0x40	/* bus communication status	*/

/* Initial start enable trigger (STCEN) */
#define	IIC0_STARTWITHSTOP			0x02
#define	IIC0_START_WITHSTOP			0x00	/* generation of a start condition upon detection of a stop condition */
#define	IIC0_START_WITHOUTSTOP		0x02	/* generation of a start condition without detecting a stop condition */

/* Communication reservation function disable bit (IICRSV) */
#define	IIC0_RESERVATION			0x01
#define	IIC0_RESERVATION_ENABLE		0x00	/* enable communication	reservation	*/	
#define	IIC0_RESERVATION_DISABLE	0x01	/* disable communication reservation */		

/*
	IIC clock selection register 0 (IICCL0)
*/
#define IICCL0_INITIALVALUE			0x00
/* Detection of	SCL0 pin level (CLD0) */
#define	IIC0_SCLLEVEL				0x20
#define	IIC0_SCL_LOW				0x00	/* clock line at low level */
#define	IIC0_SCL_HIGH				0x20	/* clock line at high level	*/

/* Detection of SDA0 pin level (DAD0) */
#define	IIC0_SDALEVEL				0x10
#define	IIC0_SDA_LOW				0x00	/* data	line at	low	level */
#define	IIC0_SDA_HIGH				0x10	/* data	line at	high level */

/* Operation mode switching (SMC0) */
#define	IIC0_OPERATIONMODE			0x08
#define	IIC0_MODE_STANDARD			0x00	/* operates	in standard	mode */
#define	IIC0_MODE_HIGHSPEED			0x08	/* operates	in high-speed mode */

/* Digital filter operation control (DFC0) */
#define	IIC0_DIGITALFILTER			0x04
#define	IIC0_FILTER_OFF				0x00	/* digital filter off */ 
#define	IIC0_FILTER_ON				0x04	/* digital filter on */

/* Operation mode switching (CL01, CL00) */
#define	IIC0_CLOCKSELECTION			0x03

/* Combine of (SMC0, CL01, CL00)*/
#define	IIC0_CLOCK0					0x00
#define	IIC0_CLOCK1					0x01
#define	IIC0_CLOCK2					0x02
#define	IIC0_CLOCK3					0x03
#define	IIC0_CLOCK4					0x08
#define	IIC0_CLOCK5					0x09
#define	IIC0_CLOCK6					0x0a
#define	IIC0_CLOCK7					0x0b

/*
	IIC	function expansion register 0 (IICX0)
*/
/* IIC clock expension (CLX0) */
#define	IIC0_CLOCKEXPENSION			0x01
#define	IIC0_EXPENSION0				0x00
#define	IIC0_EXPENSION1				0x01

/*	Operation clock	(CLX0, SMC0, CL01, CL00)

				|	IIC0_EXPENSION0	|	IIC0_EXPENSION1	|								 
	------------|-------------------|-------------------|----------------------
	IIC0_CLOCK0	|		fprs/2		|		prohibited	|	selection clock(fw)
				|		fprs/88		|					|	transfer clock
				|		normal		|					|	mode
	------------|-------------------|-------------------|----------------------
	IIC0_CLOCK1	|		fprs/2		|		prohibited	|	selection clock(fw)
				|		fprs/172	|					|	transfer clock
				|		normal		|					|	mode
	------------|-------------------|-------------------|----------------------
	IIC0_CLOCK2	|		fprs/2		|		prohibited	|	selection clock(fw)
				|		fprs/344	|					|	transfer clock
				|		normal		|					|	mode
	------------|-------------------|-------------------|----------------------
	IIC0_CLOCK3	|prohibited/fexscl0	|		prohibited	|	selection clock(fw)
				|		fw/66		|					|	transfer clock
				|		normal		|					|	mode
	------------|-------------------|-------------------|----------------------
	IIC0_CLOCK4	|		fprs/2		|		fprs/2		|	selection clock(fw)
				|		fprs/48		|		fprs/24		|	transfer clock
				|	  high speed 	|		high speed	|	mode
	------------|-------------------|-------------------|----------------------			   
	IIC0_CLOCK5	|		fprs/2		|		fprs/2		|	selection clock(fw)
				|		fprs/48		|		fprs/24		|	transfer clock
				|	  high speed	|		high speed	|	mode
	------------|-------------------|-------------------|----------------------
	IIC0_CLOCK6	|		fprs/4		|		fprs/4		|	selection clock(fw)
				|		fprs/96		|		fprs/48		|	transfer clock
				|	  high speed	|		high speed	|	mode
	------------|-------------------|-------------------|----------------------
	IIC0_CLOCK7	|prohibited/fexscl0	|		prohibited	|	selection clock(fw)
				|		fw/18		|					|	transfer clock
				|	  high speed	|					|	mode
	------------|-------------------|-------------------|----------------------
*/

#define ADDRESS_COMPLETE			0x80
#define IIC0_MASTER_FLAG_CLEAR		0x00

/*
*******************************************************************************
**  Macro define
*******************************************************************************
*/
#define IIC0_MASTERADDRESS	0x10
enum TransferMode { SEND, RECEIVE };

/*
*******************************************************************************
**  Function define
*******************************************************************************
*/
void IIC0_Init( void );
MD_STATUS IIC0_MasterSendStart( UCHAR adr, UCHAR* txbuf, USHORT txnum, UCHAR wait );
MD_STATUS IIC0_MasterReceiveStart( UCHAR adr, UCHAR* rxbuf, USHORT rxnum, UCHAR wait );
void IIC0_Stop( void );
void CALL_IIC0_MasterSendEnd( void );
void CALL_IIC0_MasterReceiveEnd( void );
void CALL_IIC0_MasterError( MD_STATUS flag );
void IIC0_MasterHandler( void );
__interrupt void MD_INTIIC0( void );
/* Start user code for definition. Do not edit comment generated here */
/* End user code for definition. Do not edit comment generated here */
#endif
