/*
*******************************************************************************
**
**  This device driver was created by Applilet2 for 78K0/Kx2
**  8-Bit Single-Chip Microcontrollers
**
**  Copyright(C) NEC Electronics Corporation 2002 - 2012
**  All rights reserved by NEC Electronics Corporation.
**
**  This program should be used on your own responsibility.
**  NEC Electronics Corporation assumes no responsibility for any losses
**  incurred by customers or third parties arising from the use of this file.
**
**  Filename :	 main.c
**  Abstract :	This file implements main function.
**  APIlib :	Applilet2 for 78K0/Kx2 V2.41 [29 Apr. 2008]
**
**  Device :	uPD78F0515_48
**
**  Compiler :	CC78K0
**
**  Creation date:	2012/07/11
**  
*******************************************************************************
*/

/*
*******************************************************************************
**  Include files
*******************************************************************************
*/
#include "macrodriver.h"
#include "Serial.h"
#include "Port.h"
#include "System.h"
#include "Timer.h"
/* Start user code for include definition. Do not edit comment generated here */
/* End user code for include definition. Do not edit comment generated here */
#include "user_define.h"

/*
*******************************************************************************
**  Global define
*******************************************************************************
*/
/* Start user code for global definition. Do not edit comment generated here */
/* End user code for global definition. Do not edit comment generated here */


/*
**-----------------------------------------------------------------------------
**
**	Abstract:
**		This function  implements main function.
**
**	Parameters:
**		None
**
**	Returns:
**		None
**
**-----------------------------------------------------------------------------
*/
#define u32 unsigned long
#define u16 unsigned int
#define u8  unsigned char
#define i8  char

/*
//==================================================================
// General Commands
//==================================================================
*/

/* STATUS bits - Used by all methods */
#define STCINT  0x01
#define ASQINT  0x02
#define RDSINT  0x04
#define RSQINT  0x08
#define ERR     0x40
#define CTS     0x80

/* POWER_UP*/
#define POWER_UP                      0x01
#define POWER_UP_IN_FUNC_FMRX         0x00
#define POWER_UP_IN_FUNC_AMRX         0x01
#define POWER_UP_IN_FUNC_FMTX         0x02
#define POWER_UP_IN_FUNC_WBRX         0x03
#define POWER_UP_IN_FUNC_QUERY        0x0F
#define POWER_UP_IN_PATCH             0x20
#define POWER_UP_IN_GPO2OEN           0x40
#define POWER_UP_IN_CTSIEN            0x80
#define POWER_UP_IN_OPMODE_RX_ANALOG  0x05
#define POWER_UP_IN_OPMODE_TX_ANALOG  0x50
#define POWER_UP_IN_XOSCEN			  0x10

/* GET_REV */
#define GET_REV 0x10

/* POWER_DOWN */
#define POWER_DOWN 0x11

/* SET_PROPERTY */
#define SET_PROPERTY 0x12

/* GET_PROPERTY */
#define GET_PROPERTY 0x13

/* GET_INT_STATUS */
#define GET_INT_STATUS 0x14

/*
//==================================================================
// AM Receive Commands
//==================================================================
*/
/* AM_TUNE_FREQ */
#define AM_TUNE_FREQ 0x40

/* AM_TUNE_STATUS */
#define AM_TUNE_STATUS           0x42
#define AM_TUNE_STATUS_IN_INTACK 0x01
#define AM_TUNE_STATUS_IN_CANCEL 0x02
#define AM_TUNE_STATUS_OUT_VALID 0x01
#define AM_TUNE_STATUS_OUT_AFCRL 0x02
#define AM_TUNE_STATUS_OUT_BTLF  0x80

#define	AM_RSQ_STATUS			0x43
/*
//==================================================================
// AM Receive Properties
//==================================================================
*/
/* AM_DEEMPHASIS */
#define AM_DEEMPHASIS      0x3100
#define AM_DEEMPHASIS_MASK 0x0001
#define AM_DEEMPHASIS_SHFT 0

/* AM_CHANNEL_FILTER*/
#define AM_CHANNEL_FILTER      0x3102
#define AM_CHANNEL_FILTER_MASK 0x0007
#define AM_CHANNEL_FILTER_SHFT 0

/* AM_SOFT_MUTE_RATE */
#define AM_SOFT_MUTE_RATE 0x3300

/* AM_SOFT_MUTE_SLOPE */
#define AM_SOFT_MUTE_SLOPE 0x3301
#define AM_SOFT_MUTE_SLOPE_MASK 0x000F
#define AM_SOFT_MUTE_SLOPE_SHFT 0

/* AM_SOFT_MUTE_MAX_ATTENUATION */
#define AM_SOFT_MUTE_MAX_ATTENUATION 0x3302
#define AM_SOFT_MUTE_MAX_ATTENUATION_MASK 0x003F
#define AM_SOFT_MUTE_MAX_ATTENUATION_SHFT 0

/* AM_SOFT_MUTE_SNR_THRESHOLD */
#define AM_SOFT_MUTE_SNR_THRESHOLD 0x3303
#define AM_SOFT_MUTE_SNR_THRESHOLD_MASK 0x003F
#define AM_SOFT_MUTE_SNR_THRESHOLD_SHFT 0

/* AM_SEEK_BAND_BOTTOM */
#define AM_SEEK_BAND_BOTTOM 0x3400

/* AM_SEEK_BAND_TOP */
#define AM_SEEK_BAND_TOP 0x3401

/* AM_SEEK_FREQ_SPACING */
#define AM_SEEK_FREQ_SPACING      0x3402
#define AM_SEEK_FREQ_SPACING_MASK 0x000F
#define AM_SEEK_FREQ_SPACING_SHFT 0

/* AM_SEEK_TUNE_SNR_THRESHOLD */
#define AM_SEEK_TUNE_SNR_THRESHOLD      0x3403
#define AM_SEEK_TUNE_SNR_THRESHOLD_MASK 0x003F
#define AM_SEEK_TUNE_SNR_THRESHOLD_SHFT 0

/* AM_SEEK_TUNE_RSSI_THRESHOLD */
#define AM_SEEK_TUNE_RSSI_THRESHOLD      0x3404
#define AM_SEEK_TUNE_RSSI_THRESHOLD_MASK 0x003F
#define AM_SEEK_TUNE_RSSI_THRESHOLD_SHFT 0

/*
//==================================================================
// General Receive Properties
//==================================================================
*/
/* RX_VOLUME */
#define RX_VOLUME      0x4000
#define RX_VOLUME_MASK 0x003F
#define RX_VOLUME_SHFT 0

/* RX_HARD_MUTE */
#define RX_HARD_MUTE 0x4001
#define RX_HARD_MUTE_RMUTE_MASK 0x0001
#define RX_HARD_MUTE_LMUTE_MASK 0x0002
#define RX_HARD_MUTE_RMUTE_SHFT 0
#define RX_HARD_MUTE_LMUTE_SHFT 1
/*
//==================================================================
// Bit Definitions for Properties
//==================================================================

/* AM_CHANNEL_FILTER_BW */
#define AM_CHANNEL_FILTER_BW_6KHZ 0
#define AM_CHANNEL_FILTER_BW_4KHZ 1
#define AM_CHANNEL_FILTER_BW_3KHZ 2
#define AM_CHANNEL_FILTER_BW_2KHZ 3
#define AM_CHANNEL_FILTER_BW_1KHZ 4
/*
//-----------------------------------------------------------------------------
// Defines
//-----------------------------------------------------------------------------
*/
#define READ    1
#define WRITE   0

#define POWERUP_TIME 110    /* Powerup delay in milliseconds*/
#define GPO_IEN				 0x0001
#define GPO_IEN_STCIEN_MASK  0x0001


/*
//==================================================================
//
//==================================================================
*/
#define	SCLK				P3.2
#define	SDIO				P3.3
#define	SENB				P3.1
#define	RSTB				P1.7

#define	LCD_CMD_WAIT		400
#define	LCD_BIT_WAIT		10

#define	FREQ_MIN			3900
#define	FREQ_MAX			12500


#define	LCD_DB4				P2.0
#define	LCD_DB5				P2.1
#define	LCD_DB6				P2.2
#define	LCD_DB7				P2.3
#define	LCD_RS				P13.0
#define	LCD_E				P14.0

#define UP_KEY				P1.0
#define	DOWN_KEY			P1.1
#define	MODE_KEY			P1.2
#define	SUB_KEY				P1.3

#define	OSCP1				P12.0
#define	OSCP2				P12.1
#define	OSCP3				P12.2
#define	SELP1				P3.0
#define	SELP2				P3.1
#define	SELP3				P3.2

#define	DAC_SCK				P7.3
#define	DAC_CS				P7.4
#define	DAC_SDI				P7.5

#define	AGC0				P7.0
#define	AGC1				P7.1
#define	AGC2				P7.2
#define	AGC3				P2.4
#define	AGC4				P2.5
#define	AGC5				P2.6

#define RSQ_INIT			10
#define	RSQ_INIT_LONG		50
#define	RSQ_INIT_AGC		20
/*
#define	RSQ_INIT_MIN		1
*/
/*
#define	MUTE_RSSI			10
*/

#define	SQ_ASNR				1
#define	AGC_RSSI_MAX		96
#define	AGC_RSSI_HI			90
#define	AGC_RSSI_LOW		80
#define AGC_SET1			18
#define	AGC_SET2			20

#define	KHZ0_MIN			22965
#define	KHZ0_MIN2			23000
#define	KHZ0_MIN3			23100
#define	KHZ0_MAX			25000
#define	KHZ0_MAX2			24900
#define	KHZ1_MIN			30300
#define	KHZ1_MIN2			30400
#define	KHZ1_MAX			32390
#define	KHZ2_MIN			32395
#define	KHZ2_MAX			34450
#define	KHZ2_MAX2			34400
#define	KHZ2_MAX3			34350

#define	VOLOUT1				100
#define	VOLOUT2				100
#define	VOLOUT3				3900
#define	VOLOUT4				3900
#define	VOLOUT5				4000
#define	VOLOUT6				4000


unsigned char	timer00_over;
u8  			cmd[8];
u8  			rsp[15];
u8  			RSSI;
u8  			STC;
u8  			BLTF;
u8  			AFCRL;
u8  			Valid;
u16 			Freq;
u8  			ASNR;
u16 			AntCap;

u8				partNumber;
char 			fwMajor;
char 			fwMinor;
u16 			patchID;
char			cmpMajor;
char			cmpMinor;
char			chipRev;


unsigned int	khz_no=KHZ0_MIN;


unsigned char	preset_no=5;
unsigned int	preset[45];
unsigned char	preset_osc[45];
unsigned char	osc;
unsigned int	tune[6];

/*
unsigned char	set_count;
unsigned char	tun_flg;
*/

unsigned char	i2c_radio_addr=0x22;
unsigned char	i2c_finish;

unsigned char	rsq_count;
unsigned char	rsq_agc;
unsigned char	filter_no=0;

unsigned char	mode=0;
unsigned char	squelch=0;
unsigned char	agc=0;
unsigned char	agc_on=0;
unsigned char	sub_mode=0;


struct {
	unsigned int PoweredUp : 1;
}Bit;


void	wait_time(unsigned int time)
{
	while(time!=0){
		time--;
	}
}

/*----------I2C,EEPROM ---------*/
void	i2c_radio_write(unsigned char number_bytes,unsigned char *data)
{
	i2c_finish = 0 ;
	IIC0_MasterSendStart(i2c_radio_addr, data, number_bytes,100);
	while (!i2c_finish){};
}
void	i2c_radio_read(unsigned char number_bytes,unsigned char *data)
{
	i2c_finish = 0 ;
	IIC0_MasterReceiveStart(i2c_radio_addr, data,number_bytes,100);
	while (!i2c_finish) {};
}
void	wait_i2c(void)
{
		wait_time(500);
}

/*----------  LCD MODULE ----------*/
void	lcd_db(unsigned char data)
{
	LCD_DB4=data&0x01;
	LCD_DB5=(data&0x02)>>1;
	LCD_DB6=(data&0x04)>>2;
	LCD_DB7=(data&0x08)>>3;
}
void	lcd_wait(void)
{
	wait_time(5000);
}
void	lcd_c_set(void)
{
	LCD_RS=0;
	LCD_E=1;
	wait_time(LCD_BIT_WAIT);
	LCD_E=0;
}
void	lcd_d_set(void)
{
	LCD_RS=1;
	LCD_E=1;
	wait_time(LCD_BIT_WAIT);
	LCD_E=0;
}
void	lcd_cmd(unsigned char data)
{
	lcd_db((data&0xf0)>>4);
	lcd_c_set();
	lcd_db(data&0x0f);
	lcd_c_set();
	wait_time(LCD_CMD_WAIT);
}
void	lcd_data(unsigned char data)
{
	lcd_db((data&0xf0)>>4);
	lcd_d_set();
	lcd_db(data&0x0f);
	lcd_d_set();
	wait_time(LCD_CMD_WAIT);
}
void	lcd_init(void)
{
	lcd_wait();
	lcd_db(3);
	lcd_c_set();
	lcd_wait();
	lcd_db(3);
	lcd_c_set();
	lcd_wait();
	lcd_db(3);
	lcd_c_set();
	lcd_wait();
	lcd_db(2);
	lcd_c_set();
	lcd_wait();
	
	lcd_cmd(0x28);
	lcd_cmd(0x08);
	lcd_cmd(0x0c);
	lcd_cmd(0x06);
}
void	lcd_clear(void)
{
	lcd_cmd(0x01);
}
void	lcd_xy(unsigned char x,unsigned char y)
{
unsigned char data;

	data=0x80;
	switch (y){
		case 0:data=0x80;break;
		case 1:data=0xc0;break;
		case 2:data=0x90;break;
		case 3:data=0xd0;break;
	}
	data=data+x;
	lcd_cmd(data);
}
void	lcd_print(char *data)
{
	while (*data!=0){
		lcd_data(*data++);
	}
}
void	lcd_su(unsigned char data)
{
	switch (data){
		case 0:lcd_print("0");break;
		case 1:lcd_print("1");break;
		case 2:lcd_print("2");break;
		case 3:lcd_print("3");break;
		case 4:lcd_print("4");break;
		case 5:lcd_print("5");break;
		case 6:lcd_print("6");break;
		case 7:lcd_print("7");break;
		case 8:lcd_print("8");break;
		case 9:lcd_print("9");break;
		case 10:lcd_print("A");break;
		case 11:lcd_print("B");break;
		case 12:lcd_print("C");break;
		case 13:lcd_print("D");break;
		case 14:lcd_print("E");break;
		case 15:lcd_print("F");break;
	}
}
void	lcd_pr_byte(unsigned char data)
{
unsigned char data2;

	data2=data%16;data=data/16;
	lcd_su(data);
	lcd_su(data2);
}
void	lcd_ini_disp(void)
{
	lcd_xy(0,0);
	lcd_print("1      M");
}

void	lcd_1keta10(unsigned char count)
{
unsigned char n0;

	n0=(unsigned char)(count%10);

	lcd_su(n0);
}

void	lcd_2keta10(unsigned char count)
{
unsigned char n0,n1;

	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);

	lcd_su(n1);
	lcd_su(n0);
}
void	lcd_4keta10(unsigned int count)
{
unsigned char n0,n1,n2,n3;

	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);count=(count-n1)/10;
	n2=(unsigned char)(count%10);count=(count-n2)/10;
	n3=(unsigned char)(count%10);

	lcd_su(n3);
	lcd_su(n2);
	lcd_su(n1);
	lcd_su(n0);
}
void	lcd_khz(unsigned int hz)
{
unsigned char n0,n1,n2,n3,n4;
unsigned int count;

	count=hz;
	n0=(unsigned char)(count%10);count=(count-n0)/10;
	n1=(unsigned char)(count%10);count=(count-n1)/10;
	n2=(unsigned char)(count%10);count=(count-n2)/10;
	n3=(unsigned char)(count%10);count=(count-n3)/10;
	n4=(unsigned char)(count%10);

	lcd_su(n4);
	lcd_su(n3);
	lcd_print(".");
	lcd_su(n2);
	lcd_su(n1);
	lcd_su(n0);
}
void	lcd_preset_no(void)
{
	lcd_xy(0,1);lcd_print("P");
	lcd_2keta10(preset_no);
}
void	lcd_osc_no(void)
{
	lcd_xy(7,1);lcd_1keta10(osc);
}
void	lcd_tune(void)
{
	lcd_xy(0,1);lcd_print("TUN");
}
void	lcd_sq(void)
{
	lcd_xy(4,1);
	if(squelch==1){
		lcd_print("S");
	}
	else{
		lcd_print(" ");
	}
}
void	lcd_agc(void)
{
	lcd_xy(5,1);
	if(agc==1){
		lcd_print("A");
	}
	else{
		lcd_print(" ");
	}
}
void	lcd_sq_sub(void)
{
	lcd_xy(4,1);
	if(squelch==1){
		lcd_print("ON  ");
	}
	else{
		lcd_print("OFF ");
	}
}
void	lcd_agc_sub(void)
{
	lcd_xy(4,1);
	if(agc==1){
		lcd_print("ON  ");
	}
	else{
		lcd_print("OFF ");
	}
}
void	lcd_sn_sub(void)
{
	lcd_xy(1,1);lcd_2keta10(RSSI);
	lcd_xy(5,1);lcd_2keta10(ASNR);
}
void	lcd_tune_sub(unsigned int nn)
{
	lcd_xy(2,1);
	if((nn>=22965)&&(nn<=23965)){lcd_print("0");lcd_xy(4,1);lcd_4keta10(tune[0]);}
	if((nn>=23970)&&(nn<=25000)){lcd_print("1");lcd_xy(4,1);lcd_4keta10(tune[1]);}
	if((nn>=30300)&&(nn<=31335)){lcd_print("2");lcd_xy(4,1);lcd_4keta10(tune[2]);}
	if((nn>=31340)&&(nn<=32390)){lcd_print("3");lcd_xy(4,1);lcd_4keta10(tune[3]);}
	if((nn>=32395)&&(nn<=33370)){lcd_print("4");lcd_xy(4,1);lcd_4keta10(tune[4]);}
	if((nn>=33375)&&(nn<=34450)){lcd_print("5");lcd_xy(4,1);lcd_4keta10(tune[5]);}
}
void	lcd_bw_sub(void)
{
	lcd_xy(4,1);
	switch(filter_no){
		case 0:	lcd_print("6kHz");break;
		case 1:	lcd_print("4kHz");break;
		case 2:	lcd_print("3kHz");break;
		case 3:	lcd_print("2kHz");break;
		case 4:	lcd_print("1kHz");break;
	}
}
/*---------- VOLTAGE  ----------*/
void	dac_out_ch1(unsigned int data)
{
unsigned int 	d0;
unsigned char	d1;
unsigned char	i;

	DAC_SCK=0;
	DAC_CS=1;
	d0=data << 4;
	d1=0x20;
	DAC_CS=0;
	
	for(i=0;i<8;i++){
		if((d1 & 0x80)==0x80){
			DAC_SDI=1;
		}
		else{
			DAC_SDI=0;
		}
		DAC_SCK=1;
		d1=d1 << 1;
		DAC_SCK=0;
	}
	
	for(i=0;i<16;i++){
		if((d0 & 0x8000)==0x8000){
			DAC_SDI=1;
		}
		else{
			DAC_SDI=0;
		}
		DAC_SCK=1;
		d0=d0 << 1;
		DAC_SCK=0;
	}
	DAC_CS=1;
	DAC_SDI=0;
}

/*----------- Si47xx ----------*/
void wait_ns(u16 ns)
{
    u8 i;

    for ( i = 1; i <= ns / 32; i++ ) /* about */
    ;
}

void wait_us(u16 us)
{
    u16 j;

    j = 8 * us-1;
    CR000=j;
    TM00_Start();
    while(timer00_over != 1); 
	TM00_Stop();
	timer00_over=0;
}

void wait_ms(u16 ms)
{
    int i;

    for ( i = 0; i < ms; i++ ) {
        wait_us(1000);
    }
}

void si47xx_reset(void)
{

    RSTB = 0;

    wait_us(100);
    RSTB = 1;
    wait_us(30);

}

void si47xx_lowRead(u8 number_bytes, u8 *data_in)
{

	i2c_radio_read(number_bytes,data_in);
	wait_i2c();
}

u8 si47xx_readStatus()
{
    u8 status;
   
    si47xx_lowRead(1, &status);

    return status;
}

void si47xx_waitForCTS()
{
    u16 i=1000;

    while (--i && !(si47xx_readStatus() & CTS))
    {
        wait_us(500);
    }
}

void si47xx_lowWrite(u8 number_bytes, u8 *data_out)
{
    i2c_radio_write(number_bytes,data_out);
    wait_i2c();
}


void si47xx_command(u8 cmd_size, u8 *cmd, u8 reply_size, u8 *reply)
{
    si47xx_waitForCTS();
    si47xx_lowWrite(cmd_size, cmd);
    si47xx_waitForCTS();
    if(reply_size)
    {
        si47xx_lowRead(reply_size, reply);
    }
}

u8 getIntStatus(void)
{
    u8  cmd[1] = {0};
    u8  rsp[1] = {0};

    cmd[0] = GET_INT_STATUS;
	si47xx_command(1, cmd, 1, rsp);
	return rsp[0];
}

void si47xx_set_property(u16 propNumber, u16 propValue)
{
    cmd[0] = SET_PROPERTY;
    cmd[1] = 0;
    cmd[2] = (u8)(propNumber >> 8);
	cmd[3] = (u8)(propNumber & 0x00FF);
    cmd[4] = (u8)(propValue >> 8);
    cmd[5] = (u8)(propValue & 0x00FF);
	si47xx_command(6, cmd, 0,  0);
}

void si47xx_getPartInformation(void)
{

	cmd[0] = GET_REV;

	si47xx_command(1, cmd, 9, rsp);

	partNumber = rsp[1];
	fwMajor  = (char)rsp[2];
	fwMinor  = (char)rsp[3];
	patchID  = (u16)(rsp[4] << 8) | (u16)rsp[5];
	cmpMajor = (char)rsp[6];
	cmpMinor = (char)rsp[7];
	chipRev  = (char)rsp[8]; 
	
}

void si47xxAMRX_powerup(void)
{
    if (Bit.PoweredUp) {
    } else {
		cmd[0] = POWER_UP;
        cmd[1] = (POWER_UP_IN_FUNC_AMRX | 0x10) ;
        cmd[2] = POWER_UP_IN_OPMODE_RX_ANALOG;
		si47xx_command(3, cmd, 8, rsp);
        wait_ms(POWERUP_TIME);
		Bit.PoweredUp = 1;
    }
}

void si47xxAMRX_powerdown(void)
{
    if(Bit.PoweredUp)
    {   
        Bit.PoweredUp = 0;
	    cmd[0] = POWER_DOWN;
		si47xx_command(1, cmd, 1, rsp);
    }
}
static void	si47xxAMRX_filer_cfg(void)
{
	switch(filter_no){
		case 0:	si47xx_set_property(AM_CHANNEL_FILTER, AM_CHANNEL_FILTER_BW_6KHZ);
				break;
		case 1:	si47xx_set_property(AM_CHANNEL_FILTER, AM_CHANNEL_FILTER_BW_4KHZ);
				break;
		case 2:	si47xx_set_property(AM_CHANNEL_FILTER, AM_CHANNEL_FILTER_BW_3KHZ);
				break;
		case 3:	si47xx_set_property(AM_CHANNEL_FILTER, AM_CHANNEL_FILTER_BW_2KHZ);
				break;
		case 4:	si47xx_set_property(AM_CHANNEL_FILTER, AM_CHANNEL_FILTER_BW_1KHZ);
				break;
	}
}

static void si47xxAMRX_hardware_cfg(void)
{
    si47xx_set_property(GPO_IEN, GPO_IEN_STCIEN_MASK);
}

static void si47xxAMRX_general_cfg(void)
{
    si47xx_set_property(AM_SEEK_TUNE_SNR_THRESHOLD, 5);
    si47xx_set_property(AM_SEEK_TUNE_RSSI_THRESHOLD, 25);
}

static void si47xxAMSWLWRX_regional_cfg(void)
{
    si47xx_set_property(AM_SEEK_FREQ_SPACING, 5);
    si47xx_set_property(AM_SEEK_BAND_BOTTOM, 3900);
    si47xx_set_property(AM_SEEK_BAND_TOP, 12500);
}
void si47xxAMSWLWRX_configure(void)
{
    si47xxAMRX_hardware_cfg();
    si47xxAMRX_general_cfg();
    si47xxAMSWLWRX_regional_cfg();
	si47xxAMRX_filer_cfg();
}

void si47xxAMSWLWRX_initialize(void)
{
	Bit.PoweredUp = 0;
    si47xx_reset();
    si47xxAMRX_powerup();
    si47xxAMSWLWRX_configure();
}

void si47xxAMRX_set_volume(u8 volume)
{
    si47xx_set_property(RX_HARD_MUTE, 0);
    si47xx_set_property(RX_VOLUME, (u16)volume & RX_VOLUME_MASK);
}

void si47xxAMRX_mute(u8 mute)
{
    if(mute)
    	si47xx_set_property(RX_HARD_MUTE, 
                                RX_HARD_MUTE_RMUTE_MASK | RX_HARD_MUTE_LMUTE_MASK);
    else
    	si47xx_set_property(RX_HARD_MUTE, 0);
}

static void amTuneFreq(u16 frequency)
{
    cmd[0] = AM_TUNE_FREQ;
    cmd[1] = 0;
    cmd[2] = (u8)(frequency >> 8);
	cmd[3] = (u8)(frequency & 0x00FF);
    cmd[4] = (u8)0;
	cmd[5] = (u8)0;
	si47xx_command(6, cmd, 1, rsp);
}

static void amTuneStatus(u8 cancel, u8 intack)
{
    cmd[0] = AM_TUNE_STATUS;

	cmd[1] = 0;
    if(cancel) cmd[1] |= AM_TUNE_STATUS_IN_CANCEL;
	if(intack) cmd[1] |= AM_TUNE_STATUS_IN_INTACK;

	si47xx_command(2, cmd, 8, rsp);

    STC    = !!(rsp[0] & STCINT);
    BLTF   = !!(rsp[1] & AM_TUNE_STATUS_OUT_BTLF);
    AFCRL  = !!(rsp[1] & AM_TUNE_STATUS_OUT_AFCRL);
    Valid  = !!(rsp[1] & AM_TUNE_STATUS_OUT_VALID);
    Freq   = ((u16)rsp[2] << 8) | (u16)rsp[3];
    RSSI   = rsp[4];
    ASNR   = rsp[5];
    AntCap = ((u16)rsp[6] << 8) | (u16)rsp[7];   
}

static void amRsqStatus(void)
{
    cmd[0] = AM_RSQ_STATUS;

	cmd[1] = 1;

	si47xx_command(2, cmd, 6, rsp);

    RSSI   = rsp[4];
    ASNR   = rsp[5];
}

static void si47xxAMRX_tune(u16 frequency)
{
    amTuneFreq(frequency);
    while (!(getIntStatus() & STCINT));
    amTuneStatus(0, 1);
}
void	tune_volt_out(unsigned int nn)
{
	if((nn>=22965)&&(nn<=23965)){dac_out_ch1(tune[0]);}
	if((nn>=23970)&&(nn<=25000)){dac_out_ch1(tune[1]);}
	if((nn>=30300)&&(nn<=31335)){dac_out_ch1(tune[2]);}
	if((nn>=31340)&&(nn<=32390)){dac_out_ch1(tune[3]);}
	if((nn>=32395)&&(nn<=33370)){dac_out_ch1(tune[4]);}
	if((nn>=33375)&&(nn<=34450)){dac_out_ch1(tune[5]);}
}
void	tune_sub(void)
{
unsigned int	khz;

	switch(mode){
		case 0:
			switch(preset_osc[preset_no]){
				case 0:	khz=preset[preset_no]-10579;
						SELP1=1;SELP2=0;SELP3=0;
						OSCP1=1;OSCP2=0;OSCP3=0;
						osc=0;
    					/*dac_out_ch1(VOLOUT1);*/
						break;
				case 1:	khz=preset[preset_no]-17950;
						SELP1=0;SELP2=1;SELP3=0;
						OSCP1=0;OSCP2=1;OSCP3=0;
						osc=1;
    					/*dac_out_ch1(VOLOUT2);*/
						break;
				case 2:	khz=preset[preset_no]-19984;
						SELP1=0;SELP2=0;SELP3=1;
						OSCP1=0;OSCP2=0;OSCP3=1;
						osc=2;
    					/*dac_out_ch1(VOLOUT3);*/
						break;
				case 3:	break;
				case 4:	break;
			}
			break;
		case 1:
			if((khz_no>=22965)&&(khz_no<=25000)){
				khz=khz_no-10579;
				SELP1=1;SELP2=0;SELP3=0;
				OSCP1=1;OSCP2=0;OSCP3=0;
				osc=0;
    			/*dac_out_ch1(VOLOUT1);*/
			}
			if((khz_no>=30300)&&(khz_no<=32390)){
				khz=khz_no-17950;
				SELP1=0;SELP2=1;SELP3=0;
				OSCP1=0;OSCP2=1;OSCP3=0;
				osc=1;
    			/*dac_out_ch1(VOLOUT2);*/
			}
			if((khz_no>=32395)&&(khz_no<=34450)){
				khz=khz_no-19984;
				SELP1=0;SELP2=0;SELP3=1;
				OSCP1=0;OSCP2=0;OSCP3=1;
				osc=2;
    			/*dac_out_ch1(VOLOUT3);*/
			}
			break;
	}
	tune_volt_out(khz_no);
    si47xxAMRX_tune(khz);
    if(RSSI>99){RSSI=99;}
    if(ASNR>99){ASNR=99;}
	/*lcd_xy(4,1);lcd_2keta10(ASNR);*/
}
unsigned int tune_volt_usub(unsigned int nn)
{
unsigned int	ret;

	ret=nn;
	if(ret<4050){ret=ret+50;}

	return ret;
}
unsigned int tune_volt_dsub(unsigned int nn)
{
unsigned int	ret;

	ret=nn;
	if(ret>50){ret=ret-50;}

	return ret;
}

void	tune_volt_up(unsigned int nn)
{
unsigned int tn;

	if((nn>=22965)&&(nn<=23965)){tn=tune_volt_usub(tune[0]);tune[0]=tn;dac_out_ch1(tune[0]);}
	if((nn>=23970)&&(nn<=25000)){tn=tune_volt_usub(tune[1]);tune[1]=tn;dac_out_ch1(tune[1]);}
	if((nn>=30300)&&(nn<=31335)){tn=tune_volt_usub(tune[2]);tune[2]=tn;dac_out_ch1(tune[2]);}
	if((nn>=31340)&&(nn<=32390)){tn=tune_volt_usub(tune[3]);tune[3]=tn;dac_out_ch1(tune[3]);}
	if((nn>=32395)&&(nn<=33370)){tn=tune_volt_usub(tune[4]);tune[4]=tn;dac_out_ch1(tune[4]);}
	if((nn>=33375)&&(nn<=34450)){tn=tune_volt_usub(tune[5]);tune[5]=tn;dac_out_ch1(tune[5]);}
}
void	tune_volt_down(unsigned int nn)
{
unsigned int tn;

	if((nn>=22965)&&(nn<=23965)){tn=tune_volt_dsub(tune[0]);tune[0]=tn;dac_out_ch1(tune[0]);}
	if((nn>=23970)&&(nn<=25000)){tn=tune_volt_dsub(tune[1]);tune[1]=tn;dac_out_ch1(tune[1]);}
	if((nn>=30300)&&(nn<=31335)){tn=tune_volt_dsub(tune[2]);tune[2]=tn;dac_out_ch1(tune[2]);}
	if((nn>=31340)&&(nn<=32390)){tn=tune_volt_dsub(tune[3]);tune[3]=tn;dac_out_ch1(tune[3]);}
	if((nn>=32395)&&(nn<=33370)){tn=tune_volt_dsub(tune[4]);tune[4]=tn;dac_out_ch1(tune[4]);}
	if((nn>=33375)&&(nn<=34450)){tn=tune_volt_dsub(tune[5]);tune[5]=tn;dac_out_ch1(tune[5]);}
}


/*---------- PRESET ---------*/
void	preset_init(void)
{
	/*------------------------------*/
	preset[0]=20000;preset_osc[0]=3;
	preset[1]=20500;preset_osc[1]=3;
	preset[2]=20975;preset_osc[2]=3;
	preset[3]=21385;preset_osc[3]=3;
	preset[4]=22680;preset_osc[4]=3;
	/*------------------------------*/
	
	preset[5]=22965;preset_osc[5]=0;
	preset[6]=23000;preset_osc[6]=0;
	preset[7]=23700;preset_osc[7]=0;
	preset[8]=23900;preset_osc[8]=0;
	preset[9]=23965;preset_osc[9]=0;
	preset[10]=24100;preset_osc[10]=0;
	preset[11]=24550;preset_osc[11]=0;
	preset[12]=25000;preset_osc[12]=0;

	/*------------------------------*/
	preset[13]=25900;preset_osc[13]=0;
	preset[14]=26100;preset_osc[14]=0;
	preset[15]=26280;preset_osc[15]=0;
	preset[16]=26281;preset_osc[16]=4;
	preset[17]=28585;preset_osc[17]=4;
	preset[18]=29967;preset_osc[18]=4;
	/*------------------------------*/

	preset[19]=30300;preset_osc[19]=1;
	preset[20]=31250;preset_osc[20]=1;
	preset[21]=31335;preset_osc[21]=1;
	preset[22]=31450;preset_osc[22]=1;
	preset[23]=32100;preset_osc[23]=1;
	preset[24]=32300;preset_osc[24]=1;
	preset[25]=32390;preset_osc[25]=1;
	preset[26]=32395;preset_osc[26]=2;
	preset[27]=32400;preset_osc[27]=2;
	preset[28]=32450;preset_osc[28]=2;
	preset[29]=32500;preset_osc[29]=2;
	preset[30]=32700;preset_osc[30]=2;
	preset[31]=33150;preset_osc[31]=2;
	preset[32]=33300;preset_osc[32]=2;
	preset[33]=33370;preset_osc[33]=2;
	preset[34]=33500;preset_osc[34]=2;
	preset[35]=33550;preset_osc[35]=2;
	preset[36]=33600;preset_osc[36]=2;
	preset[37]=33800;preset_osc[37]=2;
	preset[38]=33850;preset_osc[38]=2;
	preset[39]=34350;preset_osc[39]=2;
	preset[40]=34400;preset_osc[40]=2;
	preset[41]=34450;preset_osc[41]=2;
	
	tune[0]=VOLOUT1;tune[1]=VOLOUT2;tune[2]=VOLOUT3;
	tune[3]=VOLOUT4;tune[4]=VOLOUT5;tune[5]=VOLOUT6;
	
	
	/*------------------------------*/

}
void	preset_next(void)
{

	preset_no++;
	if(preset_no==13){preset_no=19;}
	if(preset_no>=42){preset_no=5;}

}

void	preset_back(void)
{

	if(preset_no<=5){preset_no=41;}
	else{
		if(preset_no==19){
			preset_no=12;
		}
		else{
			preset_no--;
		}
	}
}
void	khz_inc_5(void)
{
	switch(khz_no){
		case KHZ0_MAX:	khz_no=KHZ1_MIN;
						break;
		case KHZ2_MAX:	khz_no=KHZ0_MIN;
						break;
		default:		khz_no=khz_no+5;
						break;
	}
}
void	khz_dec_5(void)
{
	switch(khz_no){
		case KHZ0_MIN:	khz_no=KHZ2_MAX;
						break;
		case KHZ1_MIN:	khz_no=KHZ0_MAX;
						break;
		default:		khz_no=khz_no-5;
						break;
	}
}
void	khz_inc_100(void)
{
	if((khz_no>KHZ0_MAX2)&&(khz_no<=KHZ0_MAX)){
		khz_no=KHZ1_MIN;return;
	}
	if((khz_no>KHZ2_MAX3)&&(khz_no<=KHZ2_MAX)){
		khz_no=KHZ0_MIN2;return;
	}
	khz_no=khz_no+100;
}
void	khz_dec_100(void)
{
	if((khz_no<KHZ0_MIN3)&&(khz_no>=KHZ0_MIN)){
		khz_no=KHZ2_MAX2;return;
	}
	if((khz_no<KHZ1_MIN2)&&(khz_no>=KHZ1_MIN)){
		khz_no=KHZ0_MAX;return;
	}
	khz_no=khz_no-100;
}

/*---------- AGC ---------*/
void	agc_set(unsigned char no)
{
unsigned char	xx;
	xx=no & 0x01;if(xx==1){AGC0=1;}else{AGC0=0;}
	xx=(no & 0x02) >> 1;if(xx==1){AGC1=1;}else{AGC1=0;}
	xx=(no & 0x04) >> 2;if(xx==1){AGC2=1;}else{AGC2=0;}
	xx=(no & 0x08) >> 3;if(xx==1){AGC3=1;}else{AGC3=0;}
	xx=(no & 0x10) >> 4;if(xx==1){AGC4=1;}else{AGC4=0;}
	xx=(no & 0x20) >> 5;if(xx==1){AGC5=1;}else{AGC5=0;}
}
/*
//---------------------------------------------------------------------------
//   MAIN
//---------------------------------------------------------------------------
*/
void  main( void )
{

unsigned int	sub;
unsigned char	sw1_key;
unsigned char	sw1_rep;
unsigned char	sw1_cnt;
unsigned char	sw2_key;
unsigned char	sw2_rep;
unsigned char	sw2_cnt;
unsigned char	sw3_key;
unsigned char	sw3_rep;
unsigned char	sw3_cnt;
unsigned char	sw4_key;
unsigned char	sw4_rep;
unsigned char	sw4_cnt;


	sw1_key=0;sw1_rep=0;sw1_cnt=0;
	sw2_key=0;sw2_rep=0;sw2_cnt=0;
	sw3_key=0;sw3_rep=0;sw3_cnt=0;
	sw4_key=0;sw4_rep=0;sw4_cnt=0;

	timer00_over=0;

	Bit.PoweredUp=0;

	rsq_count=RSQ_INIT;
	rsq_agc=RSQ_INIT_AGC;

	wait_time(50000);
	agc_set(0);

	lcd_init();
	lcd_clear();

	lcd_xy(0,1);
	lcd_print("AIR-BAND");
	
	preset_init();
	
    si47xxAMSWLWRX_initialize();
    si47xx_getPartInformation();
	wait_time(50000);
	wait_time(50000);

    lcd_clear();
    lcd_xy(0,0);lcd_print("Si47  ");
    lcd_xy(4,0);lcd_2keta10(partNumber);
    lcd_xy(0,1);
    lcd_data(fwMajor);
    lcd_data(fwMinor);
    lcd_data(cmpMajor);
    lcd_data(cmpMinor);
    lcd_data(chipRev);
    
    si47xxAMRX_set_volume(63);
    
    dac_out_ch1(0);
    
	wait_time(50000);
	wait_time(50000);
	wait_time(50000);
	wait_time(50000);
	wait_time(50000);

	lcd_clear();
	lcd_ini_disp();

	khz_no=preset[preset_no];
	lcd_xy(1,0);lcd_khz(khz_no);
	lcd_preset_no();
	lcd_osc_no();lcd_sq();lcd_agc();
	tune_sub();


	if((squelch==1)&&(ASNR>=SQ_ASNR)){si47xxAMRX_set_volume(63);}


	/* Start user code. Do not edit comment generated here */
	while (1) {
	
	/* key scan 1 */
		if (UP_KEY==0){
				sw1_key=0;
		}
		else{
			sw1_key=1;
			sw1_rep=0;
			sw1_cnt=0;
		}
		/*------------------*/
		if (DOWN_KEY==0){
				sw2_key=0;
		}
		else{
			sw2_key=1;
			sw2_rep=0;
			sw2_cnt=0;
		}
		/*------------------*/
		if (MODE_KEY==0){
				sw3_key=0;
		}
		else{
			sw3_key=1;
			sw3_rep=0;
			sw3_cnt=0;
		}
		/*------------------*/
		if (SUB_KEY==0){
				sw4_key=0;
		}
		else{
			sw4_key=1;
			sw4_rep=0;
			sw4_cnt=0;
		}
		/*------------------*/


		wait_time(2000);
		
		/*-------------------*/

	/* key scan 2 */
		if ((UP_KEY==0)&&(sw1_key==0)){
			sw1_key=0;
		}
		else{
			sw1_key=1;
			sw1_rep=0;
			sw1_cnt=0;
		}
		/*------------------*/
		if ((DOWN_KEY==0)&&(sw2_key==0)){
			sw2_key=0;
		}
		else{
			sw2_key=1;
			sw2_rep=0;
			sw2_cnt=0;
		}
		/*------------------*/
		if ((MODE_KEY==0)&&(sw3_key==0)){
			sw3_key=0;
		}
		else{
			sw3_key=1;
			sw3_rep=0;
			sw3_cnt=0;
		}
		/*------------------*/
		if ((SUB_KEY==0)&&(sw4_key==0)){
			sw4_key=0;
		}
		else{
			sw4_key=1;
			sw4_rep=0;
			sw4_cnt=0;
		}
		/*------------------*/

	/* key action */
		/*------------------*/
		if((sw1_key==0)&&(sw1_rep==0)){
			switch(sub_mode){
				case 0:
					switch(mode){
						case 0:
								sw1_rep=1;
								if(squelch==1){si47xxAMRX_set_volume(0);}
								preset_next();
								khz_no=preset[preset_no];
								lcd_xy(1,0);lcd_khz(khz_no);
								lcd_preset_no();
								tune_sub();
								lcd_osc_no();lcd_sq();lcd_agc();
								if((squelch==1)&&(ASNR>=SQ_ASNR)){si47xxAMRX_set_volume(63);}
								sw1_cnt=0;
								rsq_count=RSQ_INIT;
								break;
						case 1:
								sw1_rep=1;
								if(squelch==1){si47xxAMRX_set_volume(0);}
								khz_inc_5();
								lcd_xy(1,0);lcd_khz(khz_no);
								lcd_tune();
								tune_sub();
								lcd_osc_no();lcd_sq();lcd_agc();
								if((squelch==1)&&(ASNR>=SQ_ASNR)){si47xxAMRX_set_volume(63);}
								sw1_cnt=0;
								rsq_count=RSQ_INIT;
								break;
					}
					break;
				case 1:
						sw1_rep=1;
						squelch=1;lcd_sq_sub();
						si47xxAMRX_set_volume(0);
						sw1_cnt=0;
						break;
				case 2:
						sw1_rep=1;
						agc=1;lcd_agc_sub();
						agc_on=0;
						agc_set(0);
						sw1_cnt=0;
						break;
				case 3:
						sw1_rep=1;
						sw1_cnt=0;
						break;
				case 4:
						sw1_rep=1;
						tune_volt_up(khz_no);
						lcd_tune_sub(khz_no);
						sw1_cnt=0;
						break;
				case 5:
						sw1_rep=1;
						filter_no++;
						if(filter_no>4){filter_no=0;}
						lcd_bw_sub();
						si47xxAMRX_filer_cfg();
						sw1_cnt=0;
						break;
			}
		}
		if((sw1_key==0)&&(sw1_rep==1)){
			sw1_cnt++;
			if(sw1_cnt>=30){
				switch(sub_mode){
					case 0:
						switch(mode){
							case 0:	break;
							case 1:
									if(squelch==1){si47xxAMRX_set_volume(0);}
									khz_inc_100();
									lcd_xy(1,0);lcd_khz(khz_no);
									lcd_tune();
									tune_sub();
									lcd_osc_no();lcd_sq();lcd_agc();
									if((squelch==1)&&(ASNR>=SQ_ASNR)){si47xxAMRX_set_volume(63);}
									sw1_cnt=0;
									rsq_count=RSQ_INIT;
									break;
						}
						break;
					case 1:
					case 2:
					case 3:
					case 4:
					case 5:
							break;
				}
			}
		}
		/*------------------*/
		if((sw2_key==0)&&(sw2_rep==0)){
			switch(sub_mode){
				case 0:
					switch(mode){
						case 0:
								sw2_rep=1;
								if(squelch==1){si47xxAMRX_set_volume(0);}
								preset_back();
								khz_no=preset[preset_no];
								lcd_xy(1,0);lcd_khz(khz_no);
								lcd_preset_no();
								tune_sub();
								lcd_osc_no();lcd_sq();lcd_agc();
								if((squelch==1)&&(ASNR>=SQ_ASNR)){si47xxAMRX_set_volume(63);}
								sw2_cnt=0;
								rsq_count=RSQ_INIT;
								break;
						case 1:
								sw2_rep=1;
								if(squelch==1){si47xxAMRX_set_volume(0);}
								khz_dec_5();
								lcd_xy(1,0);lcd_khz(khz_no);
								lcd_tune();
								tune_sub();
								lcd_osc_no();lcd_sq();lcd_agc();
								if((squelch==1)&&(ASNR>=SQ_ASNR)){si47xxAMRX_set_volume(63);}
								sw2_cnt=0;
								rsq_count=RSQ_INIT;
								break;
					}
					break;
				case 1:
						sw2_rep=1;
						squelch=0;lcd_sq_sub();
						si47xxAMRX_set_volume(63);
						sw2_cnt=0;
						break;
				case 2:
						sw2_rep=1;
						agc=0;lcd_agc_sub();
						agc_on=0;
						agc_set(0);
						sw2_cnt=0;
						break;
				case 3:	
						sw2_rep=1;
						sw2_cnt=0;
						break;
				case 4:
						sw2_rep=1;
						tune_volt_down(khz_no);
						lcd_tune_sub(khz_no);
						sw2_cnt=0;
						break;
				case 5:
						sw2_rep=1;
						filter_no++;
						if(filter_no>4){filter_no=0;}
						lcd_bw_sub();
						si47xxAMRX_filer_cfg();
						sw2_cnt=0;
						break;
			}
		}
		if((sw2_key==0)&&(sw2_rep==1)){
			sw2_cnt++;
			if(sw2_cnt>=30){
				switch(sub_mode){
					case 0:
						switch(mode){
							case 0:	
									break;
							case 1:
									if(squelch==1){si47xxAMRX_set_volume(0);}
									khz_dec_100();
									lcd_xy(1,0);lcd_khz(khz_no);
									lcd_tune();
									tune_sub();
									lcd_osc_no();lcd_sq();lcd_agc();
									if((squelch==1)&&(ASNR>=SQ_ASNR)){si47xxAMRX_set_volume(63);}
									sw2_cnt=0;
									rsq_count=RSQ_INIT;
									break;
						}
						break;
					case 1:
					case 2:
					case 3:
					case 4:
					case 5:
							break;
				}
			}
		}
		
		/*--------------------*/
		if((sw3_key==0)&&(sw3_rep==0)){
			switch(sub_mode){
				case 0:
					switch(mode){
						case 0:
								sw3_rep=1;
								mode=1;sub_mode=0;
								if(squelch==1){si47xxAMRX_set_volume(0);}
								lcd_xy(1,0);lcd_khz(khz_no);
								lcd_tune();
								tune_sub();
								lcd_osc_no();lcd_sq();lcd_agc();
								if((squelch==1)&&(ASNR>=SQ_ASNR)){si47xxAMRX_set_volume(63);}
								sw3_cnt=0;
								rsq_count=RSQ_INIT;
								break;
						case 1:
								sw3_rep=1;
								mode=0;sub_mode=0;
								if(squelch==1){si47xxAMRX_set_volume(0);}
								khz_no=preset[preset_no];
								lcd_xy(1,0);lcd_khz(khz_no);
								lcd_preset_no();
								tune_sub();
								lcd_osc_no();lcd_sq();lcd_agc();
								if((squelch==1)&&(ASNR>=SQ_ASNR)){si47xxAMRX_set_volume(63);}
								sw3_cnt=0;
								rsq_count=RSQ_INIT;
								break;
					}
					break;
				case 1:
				case 2:
				case 3:
				case 4:
				case 5:
					switch(mode){
						case 1:
								sw3_rep=1;
								mode=1;sub_mode=0;
								if(squelch==1){si47xxAMRX_set_volume(0);}
								lcd_xy(1,0);lcd_khz(khz_no);
								lcd_tune();lcd_xy(3,1);lcd_print("     ");
								tune_sub();
								lcd_osc_no();lcd_sq();lcd_agc();
								if((squelch==1)&&(ASNR>=SQ_ASNR)){si47xxAMRX_set_volume(63);}
								sw3_cnt=0;
								rsq_count=RSQ_INIT;
								break;
						case 0:
								sw3_rep=1;
								mode=0;sub_mode=0;
								if(squelch==1){si47xxAMRX_set_volume(0);}
								khz_no=preset[preset_no];
								lcd_xy(1,0);lcd_khz(khz_no);
								lcd_preset_no();lcd_xy(3,1);lcd_print("     ");
								tune_sub();
								lcd_osc_no();lcd_sq();lcd_agc();
								if((squelch==1)&&(ASNR>=SQ_ASNR)){si47xxAMRX_set_volume(63);}
								sw3_cnt=0;
								rsq_count=RSQ_INIT;
								break;
					}
					break;
			}
		}
		/*--------------------*/
		if((sw4_key==0)&&(sw4_rep==0)){
			switch(sub_mode){
				case 0:
				case 5:
						sw4_rep=1;
						sub_mode=1;
						lcd_xy(0,1);lcd_print("SQL=    ");
						lcd_sq_sub();
						sw4_cnt=0;
						break;
				case 1:
						sw4_rep=1;
						sub_mode=2;
						lcd_xy(0,1);lcd_print("AGC=    ");
						lcd_agc_sub();
						sw4_cnt=0;
						break;
				case 2:
						sw4_rep=1;
						sub_mode=3;
						lcd_xy(0,1);lcd_print("S   N   ");
						lcd_sn_sub();
						sw4_cnt=0;
						break;
				case 3:
						sw4_rep=1;
						sub_mode=4;
						lcd_xy(0,1);lcd_print("TN =    ");
						lcd_tune_sub(khz_no);
						sw4_cnt=0;
						break;
				case 4:
						sw4_rep=1;
						sub_mode=5;
						lcd_xy(0,1);lcd_print("BW =    ");
						lcd_bw_sub();
						sw4_cnt=0;
						break;
			}
		}


	/*-------------------------*/
		if(rsq_count>0){
			rsq_count--;
			if(rsq_count==0){
				rsq_count=RSQ_INIT;
				amRsqStatus();
    			if(RSSI>99){RSSI=99;}
    			if(ASNR>99){ASNR=99;}
    			switch(sub_mode){
    				case 0:
    					/*lcd_xy(4,1);lcd_2keta10(ASNR);*/
    					break;
    				case 1:	break;
    				case 2:	break;
    				case 3:
    					lcd_sn_sub();
    					break;
    				case 4:	break;
    			}
    			if(squelch==1){
					if(ASNR>=SQ_ASNR){
						si47xxAMRX_set_volume(63);
						rsq_count=RSQ_INIT_LONG;
					}
					else{
						si47xxAMRX_set_volume(0);
					}
				}
			}
		}
		
		if(rsq_agc>0){
			rsq_agc--;
			if(rsq_agc==0){
				rsq_agc=RSQ_INIT_AGC;
				amRsqStatus();
    			if(RSSI>99){RSSI=99;}
    			if(ASNR>99){ASNR=99;}
				if(agc==1){
					if(agc_on==0){
						if(RSSI>=AGC_RSSI_MAX){agc_set(AGC_SET2);agc_on=1;}
						if((RSSI>=AGC_RSSI_HI)&&(RSSI<AGC_RSSI_MAX)){agc_set(AGC_SET1);agc_on=1;}
					}
					else{
						if(RSSI<AGC_RSSI_LOW){agc_set(0);agc_on=0;}
					}
				}
			}
		}
		
	/*------------------------*/
	


	}
	/* End user code. Do not edit comment generated here */
}

/* Start adding user code. Do not edit comment generated here */
/* End user code adding. Do not edit comment generated here */




